--[[
Plugin that shows popup messages
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("MessageFrame", "LibSink-2.0")
if not plugin then return end

----------------------------------
-- Locals 

local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Messages")
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local LSM = LibStub("LibSharedMedia-3.0")
local Colors
local Utils = RW.Utils

local nrSlots = 5

local db

----------------------------------
-- Colors
plugin.Colors = {
	{
		name = "Posetive",
		description = "Posetive messages",
		color = {0, 1, 0},
	},
	{
		name = "Negative",
		description = "Negative messages",
		color = {1, 0.15, 0.15},
	},
	{
		name = "Information",
		description = "Informative messages",
		color = {1, 0.95, 0.64},
	},
	{
		name = "Personal",
		description = "Personal messages only shown to you",
		color = {0.33, 0.7, 1},
	}
}

----------------------------------
-- Init

function plugin:OnRegister()
	local defaults = {
		profile = {
			Width = 180,
			PluginEnabled = true,
			GrowUp = true,
			Font = "accid",
			FontSize = 23,
			FadeOutTime = 10,
			SlotHeight = 30,
			sink20OutputSink = "RW",
			ScaleUpTime = 0.3,
			ScaleDownTime = 0.6,
			DefaultSound = "RW: Bell",
			PlaySound = true,
			ChatOutput = true,
			DefaultColor = {0.39, 0.67, 0.87, 1},
			x = 0,
			y = 0,
		}
	}
	self.db = RW.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	-- Register for LibSink for message output selection
	self:SetSinkStorage(self.db.profile)
	self:RegisterSink("RW", "Raid Watch", "Messages trough RW", "AddMessage")
	
	local t = {
		type = "group",
		name = "test",
		args = {
			sink = plugin:GetSinkAce3OptionsDataTable()
		}
	}
	
	t.args.sink.guiInline = true
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RW-Messages", t)	
	
	Colors = RW:GetPlugin("Colors")
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	self:RegisterMessage("RW_ConfigMode")
	self:RegisterMessage("RW_Message")
	self:RegisterMessage("RW_DefaultsChanged")
end

function plugin:PluginDisable()
end

----------------------------------
-- Anchor

local function dragStart(self)
	self:StartMoving()
end
local function dragStop(self)
	self:StopMovingOrSizing()
	local scale = self:GetEffectiveScale() / UIParent:GetEffectiveScale()
	
	local x, y = self:GetCenter()
	x, y = x * scale, y * scale
	
	x = x - GetScreenWidth()/2
	y = y - GetScreenHeight()/2
	
	db.x = x
	db.y = y
	
	plugin:SendMessage("RW_GUIValuesChanged")
end

local function onUpdate(self, elapsed)
	local showing = false
	for i, s in next, plugin.slots do
		if s:IsShown() then
			showing = true
			s.ScaleTime = s.ScaleTime + elapsed
			-- Scale the text up untill ScaleUpTime is reached
			if s.ScaleTime < db.ScaleUpTime then
				s:SetScale(s:GetScale() + elapsed * 3)
			-- Scale down untill ScaleDownTime is reached
			elseif s.ScaleTime <= db.ScaleDownTime then
				local newScale = s:GetScale() - elapsed * 3
				s:SetScale(newScale > 0 and newScale or 0.01)
			-- Else reset the scale and text size
			else
				s:SetScale(1)
			end
			FadingFrame_OnUpdate(s)
		else
		end
	end
	if not showing then self:SetScript("OnUpdate", nil) end
end
	
function plugin:CreateAnchor()		
	local frame = Utils:CreateAnchor("MessageAnchor", UIParent, "Messages")
	local s = frame:GetEffectiveScale()
	frame:SetScript("OnDragStart", dragStart)
	frame:SetScript("OnDragStop", dragStop)
	
	self.anchor = frame
	
	self:Reset()
	
	local updateFrame = CreateFrame("Frame", nil, UIParent)	
	self.eventFrame = updateFrame
end

function plugin:Reset(var1, var2, var3)
	db = self.db.profile
	self:SetSinkStorage(self.db.profile)
	if self.anchor then
		local frame = self.anchor
		local scale = frame:GetEffectiveScale() / UIParent:GetEffectiveScale()
		frame:ClearAllPoints()
		frame:SetPoint("CENTER", UIParent, "CENTER", db.x / scale, db.y / scale)		
		frame:SetWidth(db.Width)
		
		self:RW_DefaultsChanged("RW_DefaultsChanged", "none")
	end
end

----------------------------------
-- Frame handling

-- Creates and returns one message slot
function plugin:GetMessageFrame()
	if not self.anchor then self:CreateAnchor() end
	
	-- Create message slots
	if not self.slots then
		self.slots = {}
		for i = 1, nrSlots do
			local frame = CreateFrame("Frame", "SlotFrame_"..i, self.eventFrame)
			frame:SetWidth(1)
			frame:SetHeight(db.SlotHeight)
			local str = frame:CreateFontString("RWMessageSlot_"..i, "ARTWORK")
			str:SetPoint("CENTER")
			str:SetFont(LSM:Fetch("font", db.Font), db.FontSize, "OUTLINE")
			
			local icon = frame:CreateTexture()
			icon:SetPoint("RIGHT", str, "LEFT", -4, 0)
			icon:SetWidth(db.SlotHeight - 4)
			icon:SetHeight(db.SlotHeight - 4)
			
			FadingFrame_SetFadeInTime(frame, 0.4)
			FadingFrame_SetFadeOutTime(frame, 3)
			
			frame.text = str
			frame.icon = icon
			
			frame:Hide()
			self.slots[i] = frame
		end
	end

	-- Re-arrange slot, so that slot 1 is always the next one to be used (n:th slot becomes slot nr 1)
	local slots = self.slots
	local old = slots[#slots]
	for i = #slots, 1, -1 do
		if i > 1 then
			slots[i] = slots[i-1]
		else
			slots[i] = old
		end
	end
	
	-- Position the slots according to the slots order visually
	for i, t in pairs(self.slots) do
		t:ClearAllPoints()
		if i == 1 then
			t:SetPoint(db.GrowUp and "BOTTOM" or "TOP", self.anchor, db.GrowUp and "TOP" or "BOTTOM", 0, db.GrowUp and 2 or -2)
		else
			t:SetPoint(db.GrowUp and "BOTTOM" or "TOP", self.slots[i-1], db.GrowUp and "TOP" or "BOTTOM", 0, db.GrowUp and 2 or -2)
		end
		FadingFrame_SetHoldTime(t, db.FadeOutTime)
	end	

	-- Return the "oldest" slot to be re-used
	return self.slots[1]
end

-- Internal function that is called from LibSink
function plugin:AddMessage(addon, text, r, g, b, font, size, outline, sticky, loc, icon)
	local slot = self:GetMessageFrame()
	slot.text:SetText(text)
	slot.text:SetTextColor(r, g, b)
	slot.icon:SetTexture(icon)
	slot.ScaleTime = 0
	FadingFrame_Show(slot)
	
	if not self.eventFrame:GetScript("OnUpdate") then self.eventFrame:SetScript("OnUpdate", onUpdate) end
end

----------------------------------
-- Message Handling

--- Show a new message as popup
-- @param text The text to be shown
-- @param r Red component to the color
-- @param g Green component to the color
-- @param b Blue component to the color
-- @param a Alpha component to the color
-- @param icon Optional icon to show
function plugin:RW_Message(message, text, r, g, b, a, icon, sound, mod)
	if not r or not g or not b then
		r, g, b, a = unpack(db.DefaultColor)
	end
	if db.PlaySound then
		PlaySoundFile(LSM:Fetch("sound", sound or db.DefaultSound))
	end
	if db.ChatOutput and mod then
		mod:Msg((icon and "|T"..icon..":20|t " or "")..("|cff%x%x%x"..(text or "").."|r"):format(r*255, g*255, b*255))
	end
	
	self:Pour(text or "Test message", r, g, b, nil, nil, nil, nil, nil, icon)
end

-- Enter configuration mode
function plugin:RW_ConfigMode(message, start)
	local function randomMessage()
		local spellName, rank, icon = Utils:GetRandomSpell()
		plugin:RW_Message("RW_Message", spellName ..(rank ~= "" and (" ("..rank..")" ) or "" ), math.random(), math.random(), math.random(), math.random(), icon, "None")
	end
	if not self.anchor then self:CreateAnchor() end
	if start then
		self.anchor:Show()
		self:ScheduleRepeatingTimer(randomMessage, 2)
	else
		self.anchor:Hide()
		self:CancelAllTimers()
	end
end

function plugin:RW_DefaultsChanged(message, option, value)
	if option == "Font" then
		self.db.profile.Font = value
	end
	if self.slots then
		for i, slot in pairs(self.slots) do
			slot.text:SetFont(LSM:Fetch("font", self.db.profile.Font), self.db.profile.FontSize, "OUTLINE")
		end
	end
end
---------------------------------------------

function plugin:GetOptions()
	return LB.PLUGINS, LB.ALERT_PLUGINS, L.NAME, self.GetGUI, [[Interface\Icons\INV_MISC_NOTE_02]]
end

local function changeCallback()
	plugin:Reset()
end

function plugin:GetGUI()
	local self = self or plugin
	local UIF = LibStub("AceGUI-Factory-1.0"):GetFactory("RW")
	local AceGUI = LibStub("AceGUI-3.0")
	
	local group, label, slider, check, igroup, button, color, select
	
	group = AceGUI:Create("ScrollFrame")
	group:SetLayout("Flow")
	
	label = UIF:MainTitleLabel(L.NAME)
	group:AddChild(label)
	
	igroup = UIF:InlineGroup("")
	group:AddChild(igroup)	
	
	label = UIF:NormalText(L.DESC)
	label:SetImage([[Interface\Icons\INV_MISC_NOTE_02]])
	igroup:AddChild(label)
	
	igroup = UIF:InlineGroup("")
	group:AddChild(igroup)	
	
	check = UIF:CheckBox(LB.ENABLED, db, "PluginEnabled", function() 
		if not db.PluginEnabled then
			self:Disable()
		else
			self:Enable()
		end
	end)
	igroup:AddChild(check)
	
	check = UIF:CheckBox(LB.PLAY_SOUND, db, "PlaySound")
	check:SetFullWidth(false)
	igroup:AddChild(check)
	
	select = AceGUI:Create("LSM30_Sound")
	select:SetLabel(LB.DEFAULT_SOUND)
	select:SetList()
	select:SetValue(db["DefaultSound"])
	select:SetUserData("db", db)
	select:SetUserData("key", "DefaultSound")
	select:SetCallback("OnValueChanged", function(widget, event, value)
		db["DefaultSound"] = value
	end)
	igroup:AddChild(select)
	
	check = UIF:CheckBox(LB.GROW_UP, db, "GrowUp")
	check:SetDescription(LB.GROW_UP_DESC)
	check:SetFullWidth(false)
	igroup:AddChild(check)
	
	check = UIF:CheckBox(L.CHAT_OUTPUT, db, "ChatOutput")
	check:SetDescription("Should the messages also be output to your chatwindow?")
	check:SetFullWidth(false)
	igroup:AddChild(check)
	
	color = UIF:ColorSelect(LB.DEFAULT_COLOR, db, "DefaultColor")
	igroup:AddChild(color)
	
	slider = UIF:Slider(L.FADE_OUT_TIME, db,  "FadeOutTime", 1, 15, 0.5, changeCallback)
	igroup:AddChild(slider)
	
	local sizeGroup = UIF:InlineGroup2(LB.POSITION)
	igroup:AddChild(sizeGroup)	
	
	slider = UIF:Slider("X", db, "x", 
		-floor(GetScreenWidth() * UIParent:GetEffectiveScale()), 
		floor(GetScreenWidth() * UIParent:GetEffectiveScale()),
		1, changeCallback)
	slider:SetRelativeWidth(0.5)
	sizeGroup:AddChild(slider)
	
	slider = UIF:Slider("Y", db, "y", 
		-floor(GetScreenHeight() * UIParent:GetEffectiveScale()), 
		floor(GetScreenHeight() * UIParent:GetEffectiveScale()),
		1, changeCallback)
	slider:SetRelativeWidth(0.5)
	sizeGroup:AddChild(slider)
	
	button = UIF:Button(LB.TEST, function() 
		local spellName, rank, icon = Utils:GetRandomSpell()
		plugin:SendMessage("RW_Message", spellName ..(rank ~= "" and (" ("..rank..")" ) or "" ), nil, nil, nil, nil, icon, nil, plugin)
	end)
	group:AddChild(button)
		
	button = UIF:Button(LB.RESET, function() 
		plugin.db:ResetProfile() 
	end)
	button:SetUserData("confirm", true)
	button:SetUserData("TriggerGlobalUpdate", true)
	group:AddChild(button)
	
	local sinkgroup = AceGUI:Create("SimpleGroup")
	sinkgroup:SetLayout("Fill")
	sinkgroup:SetFullWidth(true)
	group:AddChild(sinkgroup)
	LibStub("AceConfigDialog-3.0"):Open("RW-Messages", sinkgroup)
	
	group:FixScroll()
	
	return group
end

